@@ -17,8 +17,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GCC; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+the Free Software Foundation, 51 Franklin Street, Fifth Floor,
+Boston, MA 02110-1301, USA.  */
 
 
 /* This file contains the functions for converting C++ expressions
@@ -36,6 +36,7 @@ Boston, MA 02111-1307, USA.  */
 #include "convert.h"
 #include "toplev.h"
 #include "decl.h"
+#include "target.h"
 
 static tree cp_convert_to_pointer (tree, tree, bool);
 static tree convert_to_pointer_force (tree, tree);
@@ -54,7 +55,7 @@ static void warn_ref_binding (tree, tree, tree);
    narrowing is always done with a NOP_EXPR:
      In convert.c, convert_to_integer.
      In c-typeck.c, build_binary_op_nodefault (boolean ops),
-        and c_common_truthvalue_conversion.
+	and c_common_truthvalue_conversion.
      In expr.c: expand_expr, for operands of a MULT_EXPR.
      In fold-const.c: fold.
      In tree.c: get_narrower and get_unwidened.
@@ -88,7 +89,7 @@ cp_convert_to_pointer (tree type, tree expr, bool force)
       if (!COMPLETE_TYPE_P (intype))
 	{
 	  error ("can't convert from incomplete type %qT to %qT",
-                 intype, type);
+		 intype, type);
 	  return error_mark_node;
 	}
 
@@ -97,7 +98,7 @@ cp_convert_to_pointer (tree type, tree expr, bool force)
 	{
 	  if (rval == error_mark_node)
 	    error ("conversion of %qE from %qT to %qT is ambiguous",
-                   expr, intype, type);
+		   expr, intype, type);
 	  return rval;
 	}
     }
@@ -140,7 +141,7 @@ cp_convert_to_pointer (tree type, tree expr, bool force)
 	  intype_class = TREE_TYPE (intype);
 	  type_class = TREE_TYPE (type);
 
-	  same_p = same_type_p (TYPE_MAIN_VARIANT (intype_class), 
+	  same_p = same_type_p (TYPE_MAIN_VARIANT (intype_class),
 				TYPE_MAIN_VARIANT (type_class));
 	  binfo = NULL_TREE;
 	  /* Try derived to base conversion.  */
@@ -166,7 +167,7 @@ cp_convert_to_pointer (tree type, tree expr, bool force)
       if (TYPE_PTRMEMFUNC_P (type))
 	{
 	  error ("cannot convert %qE from type %qT to type %qT",
-                 expr, intype, type);
+		 expr, intype, type);
 	  return error_mark_node;
 	}
 
@@ -174,7 +175,7 @@ cp_convert_to_pointer (tree type, tree expr, bool force)
     }
   else if (TYPE_PTRMEM_P (type) && TYPE_PTRMEM_P (intype))
     {
-      tree b1; 
+      tree b1;
       tree b2;
       tree binfo;
       enum tree_code code = PLUS_EXPR;
@@ -194,12 +195,12 @@ cp_convert_to_pointer (tree type, tree expr, bool force)
       if (bk == bk_via_virtual)
 	{
 	  if (force)
-	    warning ("pointer to member cast from %qT to %qT is via"
-                     " virtual base", intype, type);
+	    warning (0, "pointer to member cast from %qT to %qT is via"
+		     " virtual base", intype, type);
 	  else
 	    {
 	      error ("pointer to member cast from %qT to %qT is"
-                     " via virtual base", intype, type);
+		     " via virtual base", intype, type);
 	      return error_mark_node;
 	    }
 	  /* This is a reinterpret cast, whose result is unspecified.
@@ -211,7 +212,7 @@ cp_convert_to_pointer (tree type, tree expr, bool force)
 	expr = cplus_expand_constant (expr);
 
       if (binfo && !integer_zerop (BINFO_OFFSET (binfo)))
-	expr = size_binop (code, 
+	expr = size_binop (code,
 			   build_nop (sizetype, expr),
 			   BINFO_OFFSET (binfo));
       return build_nop (type, expr);
@@ -235,7 +236,7 @@ cp_convert_to_pointer (tree type, tree expr, bool force)
 	    }
 	}
       error ("cannot convert %qE from type %qT to type %qT",
-             expr, intype, type);
+	     expr, intype, type);
       return error_mark_node;
     }
 
@@ -255,7 +256,7 @@ cp_convert_to_pointer (tree type, tree expr, bool force)
 	}
       else
 	expr = build_int_cst (type, 0);
-      
+
       return expr;
     }
   else if (TYPE_PTR_TO_MEMBER_P (type) && INTEGRAL_CODE_P (form))
@@ -274,7 +275,7 @@ cp_convert_to_pointer (tree type, tree expr, bool force)
 	 as a pointer.  */
       gcc_assert (GET_MODE_SIZE (TYPE_MODE (TREE_TYPE (expr)))
 		  == GET_MODE_SIZE (TYPE_MODE (type)));
-      
+
       return convert_to_pointer (type, expr);
     }
 
@@ -282,7 +283,7 @@ cp_convert_to_pointer (tree type, tree expr, bool force)
     return instantiate_type (type, expr, tf_error | tf_warning);
 
   error ("cannot convert %qE from type %qT to type %qT",
-         expr, intype, type);
+	 expr, intype, type);
   return error_mark_node;
 }
 
@@ -295,7 +296,7 @@ convert_to_pointer_force (tree type, tree expr)
 {
   tree intype = TREE_TYPE (expr);
   enum tree_code form = TREE_CODE (intype);
-  
+
   if (form == POINTER_TYPE)
     {
       intype = TYPE_MAIN_VARIANT (intype);
@@ -322,8 +323,8 @@ convert_to_pointer_force (tree type, tree expr)
 	  if (binfo)
 	    {
 	      expr = build_base_path (code, expr, binfo, 0);
-              if (expr == error_mark_node)
-                 return error_mark_node;
+	      if (expr == error_mark_node)
+		 return error_mark_node;
 	      /* Add any qualifier conversions.  */
 	      if (!same_type_p (TREE_TYPE (TREE_TYPE (expr)),
 				TREE_TYPE (type)))
@@ -363,7 +364,7 @@ build_up_reference (tree type, tree arg, int flags, tree decl)
 
       /* Process the initializer for the declaration.  */
       DECL_INITIAL (arg) = targ;
-      cp_finish_decl (arg, targ, NULL_TREE, 
+      cp_finish_decl (arg, targ, /*init_const_expr_p=*/false, NULL_TREE,
 		      LOOKUP_ONLYCONVERTING|DIRECT_BIND);
     }
   else if (!(flags & DIRECT_BIND) && ! lvalue_p (arg))
@@ -398,7 +399,7 @@ build_up_reference (tree type, tree arg, int flags, tree decl)
 /* Subroutine of convert_to_reference. REFTYPE is the target reference type.
    INTYPE is the original rvalue type and DECL is an optional _DECL node
    for diagnostics.
-   
+
    [dcl.init.ref] says that if an rvalue is used to
    initialize a reference, then the reference must be to a
    non-volatile const type.  */
@@ -407,23 +408,23 @@ static void
 warn_ref_binding (tree reftype, tree intype, tree decl)
 {
   tree ttl = TREE_TYPE (reftype);
-  
+
   if (!CP_TYPE_CONST_NON_VOLATILE_P (ttl))
     {
       const char *msg;
 
       if (CP_TYPE_VOLATILE_P (ttl) && decl)
 	  msg = "initialization of volatile reference type %q#T from"
-            " rvalue of type %qT";
+	    " rvalue of type %qT";
       else if (CP_TYPE_VOLATILE_P (ttl))
 	  msg = "conversion to volatile reference type %q#T "
-            " from rvalue of type %qT";
+	    " from rvalue of type %qT";
       else if (decl)
 	  msg = "initialization of non-const reference type %q#T from"
-            " rvalue of type %qT";
+	    " rvalue of type %qT";
       else
 	  msg = "conversion to non-const reference type %q#T from"
-            " rvalue of type %qT";
+	    " rvalue of type %qT";
 
       pedwarn (msg, reftype, intype);
     }
@@ -438,7 +439,7 @@ warn_ref_binding (tree reftype, tree intype, tree decl)
 
 tree
 convert_to_reference (tree reftype, tree expr, int convtype,
-                      int flags, tree decl)
+		      int flags, tree decl)
 {
   tree type = TYPE_MAIN_VARIANT (TREE_TYPE (reftype));
   tree intype;
@@ -446,9 +447,9 @@ convert_to_reference (tree reftype, tree expr, int convtype,
   tree rval_as_conversion = NULL_TREE;
   bool can_convert_intype_to_type;
 
-  if (TREE_CODE (type) == FUNCTION_TYPE 
+  if (TREE_CODE (type) == FUNCTION_TYPE
       && TREE_TYPE (expr) == unknown_type_node)
-    expr = instantiate_type (type, expr, 
+    expr = instantiate_type (type, expr,
 			     (flags & LOOKUP_COMPLAIN)
 			     ? tf_error | tf_warning : tf_none);
 
@@ -458,6 +459,7 @@ convert_to_reference (tree reftype, tree expr, int convtype,
   intype = TREE_TYPE (expr);
 
   gcc_assert (TREE_CODE (intype) != REFERENCE_TYPE);
+  gcc_assert (TREE_CODE (reftype) == REFERENCE_TYPE);
 
   intype = TYPE_MAIN_VARIANT (intype);
 
@@ -491,11 +493,11 @@ convert_to_reference (tree reftype, tree expr, int convtype,
 
 	  if (! real_lvalue_p (expr))
 	    warn_ref_binding (reftype, intype, decl);
-	  
+
 	  if (! (convtype & CONV_CONST)
 		   && !at_least_as_qualified_p (ttl, ttr))
 	    pedwarn ("conversion from %qT to %qT discards qualifiers",
-                     ttr, reftype);
+		     ttr, reftype);
 	}
 
       return build_up_reference (reftype, expr, flags, decl);
@@ -508,13 +510,13 @@ convert_to_reference (tree reftype, tree expr, int convtype,
 	 should be done directly (jason).  (int &)ri ---> *(int*)&ri */
 
       /* B* bp; A& ar = (A&)bp; is valid, but it's probably not what they
-         meant.  */
+	 meant.  */
       if (TREE_CODE (intype) == POINTER_TYPE
 	  && (comptypes (TREE_TYPE (intype), type,
 			 COMPARE_BASE | COMPARE_DERIVED)))
-	warning ("casting %qT to %qT does not dereference pointer",
+	warning (0, "casting %qT to %qT does not dereference pointer",
 		 intype, reftype);
-	  
+
       rval = build_unary_op (ADDR_EXPR, expr, 0);
       if (rval != error_mark_node)
 	rval = convert_force (build_pointer_type (TREE_TYPE (reftype)),
@@ -554,7 +556,7 @@ convert_from_reference (tree val)
     {
       tree t = canonical_type_variant (TREE_TYPE (TREE_TYPE (val)));
       tree ref = build1 (INDIRECT_REF, t, val);
-      
+
        /* We *must* set TREE_READONLY when dereferencing a pointer to const,
 	  so that we get the proper error message if the result is used
 	  to assign to.  Also, &* is supposed to be a no-op.  */
@@ -565,7 +567,7 @@ convert_from_reference (tree val)
       REFERENCE_REF_P (ref) = 1;
       val = ref;
     }
-  
+
   return val;
 }
 
@@ -601,6 +603,7 @@ ocp_convert (tree type, tree expr, int convtype, int flags)
 {
   tree e = expr;
   enum tree_code code = TREE_CODE (type);
+  const char *invalid_conv_diag;
 
   if (error_operand_p (e) || type == error_mark_node)
     return error_mark_node;
@@ -608,6 +611,13 @@ ocp_convert (tree type, tree expr, int convtype, int flags)
   complete_type (type);
   complete_type (TREE_TYPE (expr));
 
+  if ((invalid_conv_diag
+       = targetm.invalid_conversion (TREE_TYPE (expr), type)))
+    {
+      error (invalid_conv_diag);
+      return error_mark_node;
+    }
+
   e = integral_constant_value (e);
 
   if (IS_AGGR_TYPE (type) && (convtype & CONV_FORCE_TEMP)
@@ -627,9 +637,9 @@ ocp_convert (tree type, tree expr, int convtype, int flags)
 	   convert, which will call ocp_convert, etc.  */
 	return e;
       /* For complex data types, we need to perform componentwise
-         conversion.  */
+	 conversion.  */
       else if (TREE_CODE (type) == COMPLEX_TYPE)
-        return fold_if_not_in_template (convert_to_complex (type, e));
+	return fold_if_not_in_template (convert_to_complex (type, e));
       else if (TREE_CODE (e) == TARGET_EXPR)
 	{
 	  /* Don't build a NOP_EXPR of class type.  Instead, change the
@@ -659,10 +669,10 @@ ocp_convert (tree type, tree expr, int convtype, int flags)
     {
       tree intype = TREE_TYPE (e);
       /* enum = enum, enum = int, enum = float, (enum)pointer are all
-         errors.  */
+	 errors.  */
       if (TREE_CODE (type) == ENUMERAL_TYPE
 	  && (((INTEGRAL_OR_ENUMERATION_TYPE_P (intype)
-	        || TREE_CODE (intype) == REAL_TYPE)
+		|| TREE_CODE (intype) == REAL_TYPE)
 	       && ! (convtype & CONV_STATIC))
 	      || TREE_CODE (intype) == POINTER_TYPE))
 	{
@@ -696,11 +706,11 @@ ocp_convert (tree type, tree expr, int convtype, int flags)
 	{
 	  tree ret_val;
 	  ret_val = build_type_conversion (type, e);
-          if (ret_val)
-            return ret_val;
-          if (flags & LOOKUP_COMPLAIN)
-            error ("%q#T used where a %qT was expected", in_vtype, type);
-          return error_mark_node;
+	  if (ret_val)
+	    return ret_val;
+	  if (flags & LOOKUP_COMPLAIN)
+	    error ("%q#T used where a %qT was expected", in_vtype, type);
+	  return error_mark_node;
 	}
       return fold_if_not_in_template (convert_to_vector (type, e));
     }
@@ -752,7 +762,7 @@ ocp_convert (tree type, tree expr, int convtype, int flags)
 	   the target with the temp (see [dcl.init]).  */
 	ctor = build_user_type_conversion (type, ctor, flags);
       else
-	ctor = build_special_member_call (NULL_TREE, 
+	ctor = build_special_member_call (NULL_TREE,
 					  complete_ctor_identifier,
 					  build_tree_list (NULL_TREE, ctor),
 					  type, flags);
@@ -762,7 +772,7 @@ ocp_convert (tree type, tree expr, int convtype, int flags)
 
   if (flags & LOOKUP_COMPLAIN)
     error ("conversion from %qT to non-scalar type %qT requested",
-           TREE_TYPE (expr), type);
+	   TREE_TYPE (expr), type);
   return error_mark_node;
 }
 
@@ -780,13 +790,13 @@ ocp_convert (tree type, tree expr, int convtype, int flags)
    volatile references we do not do this interpretation, because that would
    make it impossible to ignore the reference return value from functions. We
    issue warnings in the confusing cases.
-   
+
    IMPLICIT is tells us the context of an implicit void conversion.  */
 
 tree
 convert_to_void (tree expr, const char *implicit)
 {
-  if (expr == error_mark_node 
+  if (expr == error_mark_node
       || TREE_TYPE (expr) == error_mark_node)
     return error_mark_node;
   if (!TREE_TYPE (expr))
@@ -804,85 +814,88 @@ convert_to_void (tree expr, const char *implicit)
     {
     case COND_EXPR:
       {
-        /* The two parts of a cond expr might be separate lvalues.  */
-        tree op1 = TREE_OPERAND (expr,1);
-        tree op2 = TREE_OPERAND (expr,2);
-        tree new_op1 = convert_to_void
+	/* The two parts of a cond expr might be separate lvalues.  */
+	tree op1 = TREE_OPERAND (expr,1);
+	tree op2 = TREE_OPERAND (expr,2);
+	tree new_op1 = convert_to_void
 	  (op1, (implicit && !TREE_SIDE_EFFECTS (op2)
 		 ? "second operand of conditional" : NULL));
-        tree new_op2 = convert_to_void
+	tree new_op2 = convert_to_void
 	  (op2, (implicit && !TREE_SIDE_EFFECTS (op1)
 		 ? "third operand of conditional" : NULL));
-        
+
 	expr = build3 (COND_EXPR, TREE_TYPE (new_op1),
 		       TREE_OPERAND (expr, 0), new_op1, new_op2);
-        break;
+	break;
       }
-    
+
     case COMPOUND_EXPR:
       {
-        /* The second part of a compound expr contains the value.  */
-        tree op1 = TREE_OPERAND (expr,1);
-        tree new_op1 = convert_to_void
+	/* The second part of a compound expr contains the value.  */
+	tree op1 = TREE_OPERAND (expr,1);
+	tree new_op1 = convert_to_void
 	  (op1, (implicit && !TREE_NO_WARNING (expr)
 		 ? "right-hand operand of comma" : NULL));
-        
-        if (new_op1 != op1)
+
+	if (new_op1 != op1)
 	  {
 	    tree t = build2 (COMPOUND_EXPR, TREE_TYPE (new_op1),
 			     TREE_OPERAND (expr, 0), new_op1);
 	    expr = t;
 	  }
 
-        break;
+	break;
       }
-    
+
     case NON_LVALUE_EXPR:
     case NOP_EXPR:
       /* These have already decayed to rvalue.  */
       break;
-    
+
     case CALL_EXPR:   /* We have a special meaning for volatile void fn().  */
       break;
-    
+
     case INDIRECT_REF:
       {
-        tree type = TREE_TYPE (expr);
-        int is_reference = TREE_CODE (TREE_TYPE (TREE_OPERAND (expr, 0)))
-                           == REFERENCE_TYPE;
-        int is_volatile = TYPE_VOLATILE (type);
-        int is_complete = COMPLETE_TYPE_P (complete_type (type));
-        
-        if (is_volatile && !is_complete)
-          warning ("object of incomplete type %qT will not be accessed in %s",
-                   type, implicit ? implicit : "void context");
-        else if (is_reference && is_volatile)
-          warning ("object of type %qT will not be accessed in %s",
-                   TREE_TYPE (TREE_OPERAND (expr, 0)),
-                   implicit ? implicit : "void context");
-        if (is_reference || !is_volatile || !is_complete)
-          expr = TREE_OPERAND (expr, 0);
-      
-        break;
+	tree type = TREE_TYPE (expr);
+	int is_reference = TREE_CODE (TREE_TYPE (TREE_OPERAND (expr, 0)))
+			   == REFERENCE_TYPE;
+	int is_volatile = TYPE_VOLATILE (type);
+	int is_complete = COMPLETE_TYPE_P (complete_type (type));
+
+	/* Can't load the value if we don't know the type.  */
+	if (is_volatile && !is_complete)
+	  warning (0, "object of incomplete type %qT will not be accessed in %s",
+		   type, implicit ? implicit : "void context");
+	/* Don't load the value if this is an implicit dereference, or if
+	   the type needs to be handled by ctors/dtors.  */
+	else if (is_volatile && (is_reference || TREE_ADDRESSABLE (type)))
+	  warning (0, "object of type %qT will not be accessed in %s",
+		   TREE_TYPE (TREE_OPERAND (expr, 0)),
+		   implicit ? implicit : "void context");
+	if (is_reference || !is_volatile || !is_complete || TREE_ADDRESSABLE (type))
+	  expr = TREE_OPERAND (expr, 0);
+
+	break;
       }
-    
+
     case VAR_DECL:
       {
-        /* External variables might be incomplete.  */
-        tree type = TREE_TYPE (expr);
-        int is_complete = COMPLETE_TYPE_P (complete_type (type));
-        
-        if (TYPE_VOLATILE (type) && !is_complete)
-          warning ("object %qE of incomplete type %qT will not be accessed in %s",
-                   expr, type, implicit ? implicit : "void context");
-        break;
+	/* External variables might be incomplete.  */
+	tree type = TREE_TYPE (expr);
+	int is_complete = COMPLETE_TYPE_P (complete_type (type));
+
+	if (TYPE_VOLATILE (type) && !is_complete)
+	  warning (0, "object %qE of incomplete type %qT will not be accessed in %s",
+		   expr, type, implicit ? implicit : "void context");
+	break;
       }
 
     default:;
     }
   {
     tree probe = expr;
-  
+
     if (TREE_CODE (probe) == ADDR_EXPR)
       probe = TREE_OPERAND (expr, 0);
     if (type_unknown_p (probe))
@@ -894,25 +907,32 @@ convert_to_void (tree expr, const char *implicit)
 	expr = void_zero_node;
       }
     else if (implicit && probe == expr && is_overloaded_fn (probe))
-      /* Only warn when there is no &.  */
-      warning ("%s is a reference, not call, to function %qE",
-		  implicit, expr);
+      {
+	/* Only warn when there is no &.  */
+	warning (0, "%s is a reference, not call, to function %qE",
+		 implicit, expr);
+	if (TREE_CODE (expr) == COMPONENT_REF)
+	  expr = TREE_OPERAND (expr, 0);
+      }
   }
-  
+
   if (expr != error_mark_node && !VOID_TYPE_P (TREE_TYPE (expr)))
     {
-      if (implicit && warn_unused_value && !TREE_NO_WARNING (expr))
+      if (implicit
+	  && warn_unused_value
+	  && !TREE_NO_WARNING (expr)
+	  && !processing_template_decl)
 	{
 	  /* The middle end does not warn about expressions that have
 	     been explicitly cast to void, so we must do so here.  */
 	  if (!TREE_SIDE_EFFECTS (expr))
-	    warning ("%s has no effect", implicit);
-	  else 
-	    { 
+	    warning (0, "%s has no effect", implicit);
+	  else
+	    {
 	      tree e;
 	      enum tree_code code;
 	      enum tree_code_class class;
-	      
+
 	      e = expr;
 	      /* We might like to warn about (say) "(int) f()", as the
 		 cast has no effect, but the compiler itself will
@@ -931,18 +951,20 @@ convert_to_void (tree expr, const char *implicit)
 	      class = TREE_CODE_CLASS (code);
 	      if (class == tcc_comparison
 		   || class == tcc_unary
-		   || (class == tcc_binary 
+		   || (class == tcc_binary
 		       && !(code == MODIFY_EXPR
 			    || code == INIT_EXPR
 			    || code == PREDECREMENT_EXPR
 			    || code == PREINCREMENT_EXPR
 			    || code == POSTDECREMENT_EXPR
 			    || code == POSTINCREMENT_EXPR)))
-		warning ("value computed is not used");
+		warning (0, "value computed is not used");
 	    }
 	}
       expr = build1 (CONVERT_EXPR, void_type_node, expr);
     }
+  if (! TREE_SIDE_EFFECTS (expr))
+    expr = void_zero_node;
   return expr;
 }
 
@@ -991,7 +1013,7 @@ convert_force (tree type, tree expr, int convtype)
   enum tree_code code = TREE_CODE (type);
 
   if (code == REFERENCE_TYPE)
-    return (fold_if_not_in_template 
+    return (fold_if_not_in_template
 	    (convert_to_reference (type, e, CONV_C_CAST, LOOKUP_COMPLAIN,
 				   NULL_TREE)));
 
@@ -1043,11 +1065,11 @@ build_expr_type_conversion (int desires, tree expr, bool complain)
   tree conv = NULL_TREE;
   tree winner = NULL_TREE;
 
-  if (expr == null_node 
-      && (desires & WANT_INT) 
+  if (expr == null_node
+      && (desires & WANT_INT)
       && !(desires & WANT_NULL))
-    warning ("converting NULL to non-pointer type");
-    
+    warning (0, "converting NULL to non-pointer type");
+
   basetype = TREE_TYPE (expr);
 
   if (basetype == error_mark_node)
@@ -1061,7 +1083,6 @@ build_expr_type_conversion (int desires, tree expr, bool complain)
 	  return expr;
 	/* else fall through...  */
 
-      case VECTOR_TYPE:
       case BOOLEAN_TYPE:
 	return (desires & WANT_INT) ? expr : NULL_TREE;
       case ENUMERAL_TYPE:
@@ -1070,11 +1091,28 @@ build_expr_type_conversion (int desires, tree expr, bool complain)
 	return (desires & WANT_FLOAT) ? expr : NULL_TREE;
       case POINTER_TYPE:
 	return (desires & WANT_POINTER) ? expr : NULL_TREE;
-	
+
       case FUNCTION_TYPE:
       case ARRAY_TYPE:
 	return (desires & WANT_POINTER) ? decay_conversion (expr)
-     	                                : NULL_TREE;
+					: NULL_TREE;
+
+      case VECTOR_TYPE:
+	if ((desires & WANT_VECTOR) == 0)
+	  return NULL_TREE;
+	switch (TREE_CODE (TREE_TYPE (basetype)))
+	  {
+	  case INTEGER_TYPE:
+	  case BOOLEAN_TYPE:
+	    return (desires & WANT_INT) ? expr : NULL_TREE;
+	  case ENUMERAL_TYPE:
+	    return (desires & WANT_ENUM) ? expr : NULL_TREE;
+	  case REAL_TYPE:
+	    return (desires & WANT_FLOAT) ? expr : NULL_TREE;
+	  default:
+	    return NULL_TREE;
+	  }
+
       default:
 	return NULL_TREE;
       }
@@ -1109,6 +1147,23 @@ build_expr_type_conversion (int desires, tree expr, bool complain)
 	case POINTER_TYPE:
 	  win = (desires & WANT_POINTER); break;
 
+	case VECTOR_TYPE:
+	  if ((desires & WANT_VECTOR) == 0)
+	    break;
+	  switch (TREE_CODE (TREE_TYPE (candidate)))
+	    {
+	    case BOOLEAN_TYPE:
+	    case INTEGER_TYPE:
+	      win = (desires & WANT_INT); break;
+	    case ENUMERAL_TYPE:
+	      win = (desires & WANT_ENUM); break;
+	    case REAL_TYPE:
+	      win = (desires & WANT_FLOAT); break;
+	    default:
+	      break;
+	    }
+	  break;
+
 	default:
 	  break;
 	}
@@ -1120,9 +1175,9 @@ build_expr_type_conversion (int desires, tree expr, bool complain)
 	      if (complain)
 		{
 		  error ("ambiguous default type conversion from %qT",
-                         basetype);
+			 basetype);
 		  error ("  candidate conversions include %qD and %qD",
-                         winner, cand);
+			 winner, cand);
 		}
 	      return error_mark_node;
 	    }
@@ -1180,7 +1235,7 @@ type_promotes_to (tree type)
     }
   else if (type == float_type_node)
     type = double_type_node;
-    
+
   return type;
 }
 
@@ -1193,7 +1248,7 @@ type_promotes_to (tree type)
    to TYPE.  Return the resulting expression, or error_mark_node if
    the conversion was impossible.  */
 
-tree 
+tree
 perform_qualification_conversions (tree type, tree expr)
 {
   tree expr_type;
