@@ -401,7 +401,12 @@ cxx_incomplete_type_diagnostic (tree value, tree type, int diag_type)
       break;
 
     case TEMPLATE_TYPE_PARM:
-      p_msg ("invalid use of template type parameter");
+      p_msg ("invalid use of template type parameter %qT", type);
+      break;
+
+    case BOUND_TEMPLATE_TEMPLATE_PARM:
+      p_msg ("invalid use of template template parameter %qT",
+            TYPE_NAME (type));
       break;
 
     case UNKNOWN_TYPE:
@@ -719,6 +724,15 @@ digest_init (tree type, tree init)
 
 	  return error_mark_node;
 	}
+
+      if (TREE_CODE (type) == ARRAY_TYPE
+	  && TREE_CODE (init) != CONSTRUCTOR)
+	{
+	  error ("array must be initialized with a brace-enclosed"
+		 " initializer");
+	  return error_mark_node;
+	}
+
       return convert_for_initialization (NULL_TREE, type, init,
 					 LOOKUP_NORMAL | LOOKUP_ONLYCONVERTING,
 					 "initialization", NULL_TREE, 0);
@@ -788,7 +802,10 @@ process_init_constructor_array (tree type, tree init)
 	{
 	  gcc_assert (TREE_CODE (ce->index) == INTEGER_CST);
 	  if (compare_tree_int (ce->index, i) != 0)
-	    sorry ("non-trivial designated initializers not supported");
+	    {
+	      ce->value = error_mark_node;
+	      sorry ("non-trivial designated initializers not supported");
+	    }
 	}
       else
 	ce->index = size_int (i);
@@ -838,6 +855,28 @@ process_init_constructor_array (tree type, tree init)
   return flags;
 }
 
+/* INIT is the initializer for FIELD.  If FIELD is a bitfield, mask
+   INIT so that its range is bounded by that of FIELD.  Returns the
+   (possibly adjusted) initializer.  */
+
+tree
+adjust_bitfield_initializer (tree field, tree init)
+{
+  int width;
+  tree mask;
+
+  if (!DECL_C_BIT_FIELD (field))
+    return init;
+
+  width = tree_low_cst (DECL_SIZE (field), /*pos=*/1);
+  if (width < TYPE_PRECISION (TREE_TYPE (field)))
+    {
+      mask = build_low_bits_mask (TREE_TYPE (field), width);
+      init = cp_build_binary_op (BIT_AND_EXPR, init, mask);
+    }
+  return init;
+}
+
 /* Subroutine of process_init_constructor, which will process an initializer
    INIT for a class of type TYPE. Returns the flags (PICFLAG_*) which describe
    the initializers.  */
@@ -886,11 +925,15 @@ process_init_constructor_record (tree type, tree init)
 			  || TREE_CODE (ce->index) == IDENTIFIER_NODE);
 	      if (ce->index != field
 	          && ce->index != DECL_NAME (field))
-		sorry ("non-trivial designated initializers not supported");
+		{
+		  ce->value = error_mark_node;
+		  sorry ("non-trivial designated initializers not supported");
+		}
 	    }
 
 	  gcc_assert (ce->value);
 	  next = digest_init (TREE_TYPE (field), ce->value);
+	  next = adjust_bitfield_initializer (field, next);
 	  ++idx;
 	}
       else if (TYPE_NEEDS_CONSTRUCTING (TREE_TYPE (field)))
@@ -1297,12 +1340,11 @@ build_functional_cast (tree exp, tree parms)
 
   if (! IS_AGGR_TYPE (type))
     {
-      /* This must build a C cast.  */
       if (parms == NULL_TREE)
-	parms = integer_zero_node;
-      else
-	parms = build_x_compound_expr_from_list (parms, "functional cast");
+	return cp_convert (type, integer_zero_node);
 
+      /* This must build a C cast.  */
+      parms = build_x_compound_expr_from_list (parms, "functional cast");
       return build_c_cast (type, parms);
     }
 
