@@ -1,3 +1,4 @@
+// { dg-require-namedlocale "" }
 // { dg-require-fork "" }
 // { dg-require-mkfifo "" }
 
@@ -16,7 +17,7 @@
 
 // You should have received a copy of the GNU General Public License along
 // with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
 // 27.8.1.4 Overridden virtual functions
@@ -36,8 +37,8 @@ void test01()
   using namespace std;
   using namespace __gnu_test;
 
-  locale loc_fr(__gnu_test::try_named_locale("fr_FR"));
-  locale loc_en(__gnu_test::try_named_locale("en_US"));
+  locale loc_fr(locale("fr_FR"));
+  locale loc_en(locale("en_US"));
 
   const char* name = "tmp_fifo_13171-2";
   unlink(name);
