@@ -1,3 +1,4 @@
+// { dg-do compile }
 // 1999-06-23 bkoz 
 
 // Copyright (C) 1999, 2001, 2003 Free Software Foundation, Inc.
@@ -15,7 +16,7 @@
 
 // You should have received a copy of the GNU General Public License along
 // with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
 // extension headers
@@ -35,9 +36,10 @@
 #include <ext/slist>
 #include <ext/debug_allocator.h>
 #include <ext/malloc_allocator.h>
+#include <ext/new_allocator.h>
+#include <ext/array_allocator.h>
 #include <ext/mt_allocator.h>
 #include <ext/pool_allocator.h>
 #include <ext/stdio_filebuf.h>
 #include <ext/stdio_sync_filebuf.h>
 
-int main() { }
